import { CommentData } from '@code-to-json/comments';
import { HasDocumentation, HasPosition, SerializedSourceFile, SerializedSymbol, SerializedType } from '@code-to-json/core';
import { Queue, Ref } from '@code-to-json/utils';
import { Dict } from '@mike-north/types';
export declare type FormattedTypeRef = Ref<'t'>;
export declare type FormattedSymbolRef = Ref<'s'>;
export declare type FormattedSourceFileRef = Ref<'f'>;
export interface FormatterRefRegistry {
    t: FormattedTypeRef;
    s: FormattedSymbolRef;
    f: FormattedSourceFileRef;
}
export interface SideloadedDataCollector {
    types: Queue<'t', SerializedType>;
    symbols: Queue<'s', SerializedSymbol>;
    sourceFiles: Queue<'f', SerializedSourceFile>;
}
export interface FormattedType extends FormattedEntity {
    text: string;
    flags?: string[];
    constraint?: FormattedTypeRef;
    objectFlags?: string[];
    properties?: Dict<FormattedSymbolRef>;
    baseTypes?: FormattedTypeRef[];
    isThisType?: boolean;
    thisType?: FormattedTypeRef;
    numberIndexType?: FormattedTypeRef;
    stringIndexType?: FormattedTypeRef;
    defaultType?: FormattedTypeRef;
    callSignatures?: FormattedSignature[];
    constructorSignatures?: FormattedSignature[];
    typeParameters?: FormattedTypeRef[];
    libName?: string;
}
export interface FormattedSymbol extends FormattedEntity, HasDocumentation, HasPosition<FormattedSourceFileRef> {
    name: string;
    documentation?: CommentData;
    flags?: string[];
    external?: boolean;
    modifiers?: string[];
    decorators?: string[];
    heritageClauses?: string[];
    exports?: Dict<FormattedSymbolRef>;
    globalExports?: Dict<FormattedSymbolRef>;
    members?: Dict<FormattedSymbolRef>;
    properties?: Dict<FormattedSymbolRef>;
    jsDocTags?: Array<{
        name: string;
        text?: string;
    }>;
    type?: FormattedTypeRef;
}
export interface FormattedSignature {
    parameters?: Array<{
        name: string;
        type?: FormattedTypeRef;
    }>;
    typeParameters?: FormattedTypeRef[];
    returnType?: FormattedTypeRef;
}
export interface FormattedSourceFile extends FormattedEntity, Pick<FormattedSymbol, 'documentation' | 'exports'> {
    pathInPackage: string;
    moduleName: string;
    extension: string | null;
    isDeclarationFile: boolean;
    referencedFiles?: string[];
}
export interface FormattedEntity {
    id: string;
}
//# sourceMappingURL=types.d.ts.map